"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const chokidar = require("chokidar");
const path_1 = require("path");
const create_file_1 = require("./create-file");
const get_path_info_1 = require("./get-path-info");
/**
 * Watches the filesystem and yields any changes that are detected.
 */
function watch(config) {
    let watcher;
    return {
        /**
         * Starts watching the filesystem for changes.
         */
        async startWatching(context) {
            let path = await get_path_info_1.getPathInfo(config, context);
            watcher = new Watcher(path, config, context);
            return watcher.iterable;
        },
        /**
         * Stops watching the filesystem for changes.
         */
        async stopWatching() {
            if (watcher) {
                let promise = watcher.dispose();
                watcher = undefined;
                await promise;
            }
        }
    };
}
exports.watch = watch;
/**
 * Internal class that wraps the filesystem watch functionality.
 */
class Watcher {
    constructor(path, config, context) {
        this.disposed = false;
        this.chokidar = createChokidar(path, config);
        this.output = new utils_1.IterableWriter();
        this.config = config;
        this.context = context;
        this.dir = path.dir;
        this.chokidar.on("add", this.changeDetected.bind(this, "created" /* Created */));
        this.chokidar.on("change", this.changeDetected.bind(this, "modified" /* Modified */));
        this.chokidar.on("unlink", this.changeDetected.bind(this, "deleted" /* Deleted */));
        this.chokidar.on("error", this.errorHandler.bind(this));
    }
    get iterable() {
        return this.output.iterable;
    }
    /**
     * Disposes the watcher. It cannot be used after this.
     */
    async dispose() {
        if (!this.disposed) {
            this.disposed = true;
            // Don't await on this promise, since we don't know or care whether
            // the rest of the output stream will be read.
            // tslint:disable-next-line: no-floating-promises
            this.output.end();
            await this.chokidar.close();
        }
    }
    /**
     * A filesystem change was detected within the directory path.
     */
    async changeDetected(change, path, stats) {
        try {
            this.context.log.debug(`Change detected: ${change} ${path}`, { change, dir: this.dir, path });
            let absolutePath = path_1.join(this.dir, path);
            let file = create_file_1.createChangedFile(path, absolutePath, stats, change);
            if (this.config.filter(file, this.context)) {
                if (change !== "deleted" /* Deleted */) {
                    file.contents = await this.config.fs.promises.readFile(path_1.join(this.dir, path));
                }
                await this.output.write(file);
            }
        }
        catch (error) {
            await this.errorHandler(error);
        }
    }
    async errorHandler(error) {
        if (!this.disposed) {
            // Re-throw the error into the iterable
            await this.output.throw(error);
        }
    }
}
/**
 * Creates a Chokidar filesystem watcher.
 */
function createChokidar(path, config) {
    let whatToWatch, disableGlobbing;
    if (typeof config.filterCriteria === "string") {
        // The filter criteria is a glob pattern, so pass it along to Chokidar.
        whatToWatch = globify(path.dir, config.filterCriteria);
        disableGlobbing = false;
    }
    else {
        whatToWatch = path.dir;
        disableGlobbing = true;
    }
    return chokidar.watch(whatToWatch, {
        ignoreInitial: true,
        cwd: path.dir,
        disableGlobbing,
        alwaysStat: true,
        depth: config.depth,
    });
}
/**
 * Combines the specified directory path and glob pattern.
 */
function globify(dir, glob) {
    if (process.platform === "win32") {
        dir = dir.replace(/\\/g, "/");
    }
    return path_1.posix.join(dir, glob);
}
//# sourceMappingURL=watch.js.map